from llm_bridge import *

# Standard JSON Schema
structured_output_schema = {
  "$schema": "https://json-schema.org/draft/2020-12/schema",
  "$id": "https://example.com/product.schema.json",
  "title": "Product",
  "description": "A product from Acme's catalog",
  "type": "object",
  "properties": {
    "productId": {
      "description": "The unique identifier for a product",
      "type": "integer"
    },
    "productName": {
      "description": "Name of the product",
      "type": "string"
    },
    "price": {
      "description": "The price of the product",
      "type": "number",
      "exclusiveMinimum": 0
    },
    "tags": {
      "description": "Tags for the product",
      "type": "array",
      "items": {
        "type": "string"
      },
      "minItems": 1,
      "uniqueItems": True
    }
  },
  "required": [
    "productId",
    "productName",
    "price"
  ],
  "additionalProperties": False
}

# OpenAI Responses API JSON Schema
# structured_output_schema = {
#   "$schema": "https://json-schema.org/draft/2020-12/schema",
#   "$id": "https://example.com/product.schema.json",
#   "title": "Product",
#   "description": "A product from Acme's catalog",
#   "type": "object",
#   "properties": {
#     "productId": {
#       "description": "The unique identifier for a product",
#       "type": "integer"
#     },
#     "productName": {
#       "description": "Name of the product",
#       "type": "string"
#     },
#     "price": {
#       "description": "The price of the product",
#       "type": "number",
#       "exclusiveMinimum": 0
#     },
#     "tags": {
#       "description": "Tags for the product",
#       "type": "array",
#       "items": {
#         "type": "string"
#       },
#       "minItems": 1,
#     }
#   },
#   "required": [
#     "productId",
#     "productName",
#     "price",
#     "tags"
#   ],
#   "additionalProperties": False
# }

structured_output_schema = None

messages = [
    Message(
        role=Role.System,
        contents=[
            Content(type=ContentType.Text, data="You are a helpful assistant."),
            # Content(type=ContentType.File, data="https://www.windsnow1025.com/minio/windsnow/uploads/1/1746209841847-A%20Tutorial%20on%20Spectral%20Clustering.pdf")
        ]
    ),
    # Message(
    #     role=Role.User,
    #     contents=[
    #         Content(type=ContentType.Text, data="Hello")
    #     ]
    # ),
    # Message(
    #     role=Role.Assistant,
    #     contents=[
    #         Content(type=ContentType.Text, data="Hello! How can I assist you today?")
    #     ]
    # ),
    Message(
        role=Role.User,
        contents=[
            # Simple Question
            # Content(type=ContentType.Text, data="What's this?"),

            # Thinking
            # Content(type=ContentType.Text, data="Explain the concept of Occam's Razor and provide a simple, everyday example."),

            # Web Search
            Content(type=ContentType.Text, data="What's the weather in NYC today?"),

            # Image Understanding
            # Content(type=ContentType.File, data="https://www.gstatic.com/webp/gallery3/1.png"),
            # Content(type=ContentType.Text, data="What is in this image?"),

            # Image Generation
            # Content(type=ContentType.Text, data="Please generate an image of a cat."),

            # URL Context
            # Content(type=ContentType.Text, data="What is in https://www.windsnow1025.com/"),

            # Code Execution
            # Content(type=ContentType.Text, data="What is the sum of the first 50 prime numbers? Generate and run code for the calculation, and make sure you get all 50."),

            # File Output
            # Content(type=ContentType.Text, data="Create a matplotlib visualization and save it as output.png"),

            # Structured Output
            # Content(type=ContentType.Text, data="Please generate a product."),
        ]
    ),
    # Message(
    #     role=Role.User,
    #     contents=[
    #         Content(type=ContentType.File, data="https://www.windsnow1025.com/minio/windsnow/uploads/1/1769429581512-Test.txt"),
    #         # Content(type=ContentType.File, data="https://www.windsnow1025.com/minio/windsnow/uploads/1/1746208707489-image.png"),
    #         # Content(type=ContentType.File, data="https://www.windsnow1025.com/minio/windsnow/uploads/1/1746209841847-A%20Tutorial%20on%20Spectral%20Clustering.pdf"),
    #         # Content(type=ContentType.File, data="https://www.windsnow1025.com/minio/windsnow/uploads/1/1746212253473-file_example_MP3_700KB.mp3"),
    #         # Content(type=ContentType.File, data="https://www.windsnow1025.com/minio/windsnow/uploads/1/1746212980820-file_example_MP4_480_1_5MG.mp4"),
    #         Content(type=ContentType.Text, data="What's this?"),
    #     ]
    # ),
]

# See /llm_bridge/resources/model_prices.json for available models
# model = "gpt-5.4-mini"
# model = "gpt-5.2"
# model = "gpt-5.1"
# model = "gpt-5.4-pro"
# model = "gpt-5"
# model = "gpt-4.1"
# model = "gpt-5-codex"
# model = "gemini-3-pro-preview"
# model = "gemini-3-pro-image-preview"
# model = "gemini-3.1-flash-image-preview"
# model = "gemini-3-flash-preview"
# model = "claude-sonnet-4-6"
model = "grok-4.20-reasoning"

# api_type = "Vertex AI"
# api_type = "Google AI Studio Free Tier"
# api_type = "Google AI Studio"
# api_type = "OpenAI"
# api_type = "OpenAI-Azure"
# api_type = "OpenAI-GitHub"
# api_type = "Claude"
api_type = "Grok"

temperature = 0
stream = True
# stream = False
thought = True
# thought = False
# web_search = True
web_search = False
# code_execution = True
code_execution = False



import logging
import re
from typing import AsyncGenerator

import httpx
import openai
from fastapi import HTTPException
from openai import APIStatusError, AsyncStream
from openai.types.responses import ResponseStreamEvent

from llm_bridge.client.implementations.openai.openai_responses_response_handler import \
    process_openai_responses_stream_response
from llm_bridge.client.implementations.openai.openai_token_couter import count_openai_responses_input_tokens, \
    count_openai_output_tokens
from llm_bridge.client.model_client.openai_client import OpenAIClient
from llm_bridge.type.chat_response import ChatResponse
from llm_bridge.type.serializer import serialize


async def generate_chunk(
        stream: AsyncStream[ResponseStreamEvent],
        input_tokens: int,
) -> AsyncGenerator[ChatResponse, None]:
    try:
        async for event in stream:
            chat_response = await process_openai_responses_stream_response(event)
            output_tokens = count_openai_output_tokens(chat_response)
            yield ChatResponse(
                text=chat_response.text,
                thought=chat_response.thought,
                code=chat_response.code,
                code_output=chat_response.code_output,
                files=chat_response.files,
                input_tokens=input_tokens,
                output_tokens=output_tokens,
            )
    except Exception as e:
        logging.exception(e)
        yield ChatResponse(error=repr(e))


class StreamOpenAIResponsesClient(OpenAIClient):
    async def generate_stream_response(self) -> AsyncGenerator[ChatResponse, None]:
        try:
            logging.info(f"messages: {self.messages}")

            input_tokens = count_openai_responses_input_tokens(
                messages=self.messages
            )

            stream: AsyncStream[ResponseStreamEvent] = await self.client.responses.create(
                model=self.model,
                reasoning=self.reasoning,
                input=serialize(self.messages),
                temperature=self.temperature,
                stream=True,
                tools=self.tools,
                include=self.include,
                text=self.text,
            )

        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            text = e.response.text
            raise HTTPException(status_code=status_code, detail=text)
        except openai.BadRequestError as e:
            status_code = e.status_code
            text = e.message
            raise HTTPException(status_code=status_code, detail=text)
        except APIStatusError as e:
            status_code = e.status_code
            text = e.message
            raise HTTPException(status_code=status_code, detail=text)
        except Exception as e:
            logging.exception(e)
            match = re.search(r'\d{3}', str(e))
            if match:
                error_code = int(match.group(0))
            else:
                error_code = 500

            raise HTTPException(status_code=error_code, detail=str(e))

        async for chunk in generate_chunk(stream, input_tokens):
            yield chunk


import pytest

from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type


@pytest.mark.asyncio
async def test_get_file_type_with_extension():
    file_type, sub_type = await get_file_type("https://example.com/1767243600000-markdown.md")
    assert file_type == "text"
    assert sub_type == "code"


@pytest.mark.asyncio
async def test_get_file_type_pdf():
    file_type, sub_type = await get_file_type("https://example.com/1767243600000-document.pdf")
    assert file_type == "text"
    assert sub_type == "pdf"


@pytest.mark.asyncio
async def test_get_file_type_dockerfile():
    file_type, sub_type = await get_file_type("https://example.com/1767243600000-Dockerfile")
    assert file_type == "text"
    assert sub_type == "code"



from openai.types.responses import Response, ResponseOutputItem, ResponseOutputMessage, \
    ResponseOutputText, ResponseReasoningItem
from openai.types.responses import ResponseStreamEvent, ResponseReasoningSummaryTextDeltaEvent, ResponseTextDeltaEvent, \
    ResponseCodeInterpreterCallCodeDeltaEvent, ResponseImageGenCallPartialImageEvent, ResponseOutputItemDoneEvent, \
    ResponseCodeInterpreterToolCall
from openai.types.responses.response_code_interpreter_tool_call import Output, OutputLogs, OutputImage
from openai.types.responses.response_output_item import ImageGenerationCall

from llm_bridge.client.implementations.openai.openai_token_couter import count_openai_output_tokens
from llm_bridge.logic.chat_generate.media_processor import get_base64_content_from_url
from llm_bridge.type.chat_response import ChatResponse, File


async def process_code_interpreter_outputs(interpreter_outputs: list[Output]) -> tuple[str, list[File]]:
    code_output: str = ""
    files: list[File] = []

    for interpreter_output in interpreter_outputs:
        if interpreter_output.type == "logs":
            output_logs: OutputLogs = interpreter_output
            code_output += output_logs.logs
        if interpreter_output.type == "image":
            output_image: OutputImage = interpreter_output
            data, _ = await get_base64_content_from_url(output_image.url)
            file = File(
                name="code_interpreter_call_output.png",
                data=data,
                type="image/png",
            )
            files.append(file)

    return code_output, files


async def process_openai_responses_non_stream_response(
        response: Response,
        input_tokens: int,
) -> ChatResponse:

    output_list: list[ResponseOutputItem] = response.output

    text: str = ""
    thought: str = ""
    code: str = ""
    code_output: str = ""
    files: list[File] = []

    for output in output_list:
        if output.type == "message":
            output_message: ResponseOutputMessage = output
            for content in output_message.content:
                if content.type == "output_text":
                    output_text: ResponseOutputText = content
                    text += output_text.text
        elif output.type == "reasoning":
            reasoning_item: ResponseReasoningItem = output
            for summary_delta in reasoning_item.summary:
                thought += summary_delta.text
        elif output.type == "code_interpreter_call":
            code_interpreter_tool_call: ResponseCodeInterpreterToolCall = output
            if interpreter_code := code_interpreter_tool_call.code:
                code += interpreter_code
            if interpreter_outputs := code_interpreter_tool_call.outputs:
                interpreter_code_output, interpreter_files = await process_code_interpreter_outputs(interpreter_outputs)
                code_output += interpreter_code_output
                files.extend(interpreter_files)
        elif output.type == "image_generation_call":
            image_generation_call: ImageGenerationCall = output
            file = File(
                name="image_generation_call_output.png",
                data=image_generation_call.result,
                type="image/png",
            )
            files.append(file)

    chat_response = ChatResponse(text=text, files=files)
    output_tokens = count_openai_output_tokens(chat_response)
    return ChatResponse(
        text=text,
        thought=thought,
        code=code,
        code_output=code_output,
        files=files,
        input_tokens=input_tokens,
        output_tokens=output_tokens,
    )


async def process_openai_responses_stream_response(event: ResponseStreamEvent) -> ChatResponse:
    text: str = ""
    thought: str = ""
    code: str = ""
    code_output: str = ""
    files: list[File] = []

    if event.type == "response.output_text.delta":
        text_delta_event: ResponseTextDeltaEvent = event
        text = text_delta_event.delta
    elif event.type == "response.reasoning_summary_text.delta":
        reasoning_summary_text_delta_event: ResponseReasoningSummaryTextDeltaEvent = event
        thought = reasoning_summary_text_delta_event.delta
    elif event.type == "response.code_interpreter_call_code.delta":
        code_interpreter_call_code_delta_event: ResponseCodeInterpreterCallCodeDeltaEvent = event
        code = code_interpreter_call_code_delta_event.delta
    elif event.type == "response.output_item.done":
        output_item_done_event: ResponseOutputItemDoneEvent = event
        if output_item_done_event.item.type == "code_interpreter_call":
            code_interpreter_tool_call: ResponseCodeInterpreterToolCall = output_item_done_event.item
            if interpreter_outputs := code_interpreter_tool_call.outputs:
                interpreter_code_output, interpreter_files = await process_code_interpreter_outputs(interpreter_outputs)
                code_output += interpreter_code_output
                files.extend(interpreter_files)
    elif event.type == "response.image_generation_call.partial_image":
        image_gen_call_partial_image_event: ResponseImageGenCallPartialImageEvent = event
        file = File(
            name="generated_image.png",
            data=image_gen_call_partial_image_event.partial_image_b64,
            type="image/png",
        )
        files.append(file)

    chat_response = ChatResponse(
        text=text,
        thought=thought,
        code=code,
        code_output=code_output,
        files=files,
    )
    return chat_response



import mimetypes
import os
import re
from pathlib import PurePosixPath

from llm_bridge.logic.file_fetch import fetch_file_data
from llm_bridge.logic.message_preprocess.code_file_extensions import code_file_extensions


def is_file_type_supported(file_name: str) -> bool:
    if file_name.endswith(('.doc', '.xls', '.ppt')):
        return False
    else:
        return True


async def get_file_type(file_url: str) -> tuple[str, str]:
    file_name: str = get_filename_without_timestamp(file_url)

    # Treat filenames without an extension as their own extension
    suffix: str = PurePosixPath(file_name).suffix.lower()
    file_extension: str = suffix if suffix else '.' + file_name.lower()

    if file_extension in code_file_extensions:
        return 'text', 'code'
    if file_extension == '.pdf':
        return 'text', 'pdf'
    if file_extension == '.docx':
        return 'text', 'word'
    if file_extension == '.xlsx':
        return 'text', 'excel'
    if file_extension == '.pptx':
        return 'text', 'powerpoint'
    # At present, unable to differentiate between audio and video files. Since web recordings are uploaded in webm format, treat webm files as audio by default.
    if file_extension == '.webm':
        return 'audio', 'webm'

    mime_type, _ = mimetypes.guess_type(file_name)
    if not mime_type:
        _, mime_type = await fetch_file_data(file_url)

    if mime_type:
        return mime_type.split('/')[0], mime_type.split('/')[1]

    return 'unknown', 'unknown'


def get_filename_without_timestamp(file_url: str) -> str:
    base_name = PurePosixPath(file_url).name
    match = re.search(r'-(.+)', base_name)
    if match:
        return match.group(1)
    else:
        return base_name



import logging
import re
from typing import AsyncGenerator

import httpx
import openai
from fastapi import HTTPException
from openai import APIStatusError, AsyncStream
from openai.types.chat import ChatCompletionChunk

from llm_bridge.client.implementations.openai.openai_token_couter import count_openai_input_tokens, \
    count_openai_output_tokens
from llm_bridge.client.model_client.openai_client import OpenAIClient
from llm_bridge.type.chat_response import ChatResponse
from llm_bridge.type.serializer import serialize


def process_delta(completion_delta: ChatCompletionChunk) -> str:
    # Necessary for Azure
    if not completion_delta.choices:
        return ""

    content_delta = completion_delta.choices[0].delta.content
    if not content_delta:
        content_delta = ""
    logging.debug(f"chunk: {content_delta}")
    return content_delta


async def generate_chunk(
        completion: AsyncStream[ChatCompletionChunk],
        input_tokens: int,
) -> AsyncGenerator[ChatResponse, None]:
    try:
        async for completion_delta in completion:
            content_delta = process_delta(completion_delta)
            chat_response = ChatResponse(text=content_delta)
            output_tokens = count_openai_output_tokens(chat_response)
            yield ChatResponse(
                text=content_delta,
                input_tokens=input_tokens,
                output_tokens=output_tokens,
            )
    except Exception as e:
        logging.exception(e)
        yield ChatResponse(error=repr(e))


class StreamOpenAIClient(OpenAIClient):
    async def generate_stream_response(self) -> AsyncGenerator[ChatResponse, None]:
        try:
            logging.info(f"messages: {self.messages}")

            input_tokens = count_openai_input_tokens(self.messages)

            completion = await self.client.chat.completions.create(
                messages=serialize(self.messages),
                model=self.model,
                temperature=self.temperature,
                stream=True
            )
        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            text = e.response.text
            raise HTTPException(status_code=status_code, detail=text)
        except openai.BadRequestError as e:
            status_code = e.status_code
            text = e.message
            raise HTTPException(status_code=status_code, detail=text)
        except APIStatusError as e:
            status_code = e.status_code
            text = e.message
            raise HTTPException(status_code=status_code, detail=text)
        except Exception as e:
            logging.exception(e)
            match = re.search(r'\d{3}', str(e))
            if match:
                error_code = int(match.group(0))
            else:
                error_code = 500

            raise HTTPException(status_code=error_code, detail=str(e))

        async for chunk in generate_chunk(completion, input_tokens):
            yield chunk



from typing import AsyncGenerator

from llm_bridge.type.chat_response import ChatResponse


class ChatClient:
    async def generate_non_stream_response(self) -> ChatResponse:
        raise NotImplementedError
    async def generate_stream_response(self) -> AsyncGenerator[ChatResponse, None]:
        raise NotImplementedError
